(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports) :
    typeof define === 'function' && define.amd ? define(['exports'], factory) :
    (global = global || self, factory(global.funprog = {}));
}(this, function (exports) { 'use strict';

    const not = async (x) => {
         var operand = await x;
        return !operand
     };

    const identity = x => x;

    // predicates
    const isEven = x => x % 2 === 0;
    const isGreaterThan = x => y => y > x ? y : 0;
    const digitize = x => x ? 1 : 0;

    const modulus = val => x => x % val;

    const select = property => x => x[property];

    function apply (x, f) {
        return f(x)
    }

    function compose (...funcs) {
        return x => funcs.reduceRight(apply, x)
    }

    function concat (xs, val) {
        return xs.concat(val)
    }

    async function transduceAsyncIterator (transform, reducerfunction, init, asynciterator) {
        var reducer = transform(reducerfunction);
        var n = null;
        do {
            n = await asynciterator.next();
            if (!n.done) {
            var v = n.value;
            init = await reducer(init, v);
            }
        } while (!n.done)
        return init
    }

    async function transduceAsyncHasNextIterator (transform, reducerfunction, init, asynchasnextiterator) {
        var reducer = transform(reducerfunction);
        var n = null;
        var r = null;
        do {
            n = await asynchasnextiterator.hasNext();
            if (n) {
                var v = await asynchasnextiterator.next();
                r = await reducer(init, v);
                if (r) {
                    init = r;
                }
            }
        } while (r && n)
        return init
    }

    async function transduceArray (xf, rf, init, xs) {
        // call reduce on the data structure internally (abstract it away)
        // pass the rf to the composed transformation
        // pass in the initial value
        var xrf = await xf(rf);
        return xs.reduce(xrf, init)
    }

    // generalize the 'mapping' concept, without the concat...
    function mapping (f) {
        return function (rf) {
          // this takes 2 things and makes them 1
          return async (acc, val) => {
            var m = await f(val);
            return rf(acc, m) // <-- rf replaces 'concat'
          }
        }
      }

      // generalize the 'filtering' concept, without the concat...
      function filtering (p) {
        return function (rf) {
          // this takes 2 things and makes them 1
          return async (acc, val) => {
            var pred = (await p(val));
            return pred ? rf(acc, val) : acc // <-- rf replaces 'concat'
          }
        }
      }

    function take (cnt) {
      //assert(cnt && cnt > 0)
      var count = cnt;
      return function (rf) {
        // this takes 2 things and makes them 1
        return async (acc, val) => {
          if ((--count) < 0) {
            return acc
          } else {
            return rf(acc, val)
          }
        }
      }
    }

    function skip (cnt) {
      //assertMod(cnt && cnt >= 0)
      var count = 0;
      return function (rf) {
        // this takes 2 things and makes them 1
        return async (acc, val) => {
          if (count++ < cnt) {
            return acc
          } else {
            return rf(acc, val)
          }
        }
      }
    }

    function eventing (p) {
      var latest = null;
      var sequence = -1;
      return function (rf) {
        return async (acc, val) => {
          sequence++;
          var pred = (await p(val));
          if (pred && !latest) {
            latest = {
              start: val.time ? val.time : sequence,
              end: val.time ? val.time : sequence
            };
            return acc
          } else if (pred && latest) {
            latest.end = val.time ? val.time : sequence;
            return acc
          } else if (!pred && latest) {
            var next = latest;
            latest = null;
            return rf(acc, next)
          } else {
            return acc
          }
        }
      }
    }

    async function * makeAsyncRangeIterator (start = 0, end = Infinity, step = 1) {
        let iterationCount = 0;
        for (let i = start; i < end; i += step) {
            iterationCount++;
            yield new Promise((resolve, reject) => {
                setTimeout(() => {
                    resolve(i);
                }, 10);
            });
        }
        return iterationCount
      }

    function makeAsyncHasNextRangeIterator (start = 0, end = Infinity, step = 1) {
        let iterationCount = start;
        return {
            hasNext: function () {
                return iterationCount < end
            },
            next: function () {
                return new Promise((resolve, reject) => {
                    setTimeout(() => {
                        resolve(iterationCount++);
                    }, 10);
                })
            }
        }
    }

    function makeArrayIterator (dataArray) {
        let iterationCount = 0;
        return {
            hasNext: function () {
                return iterationCount < dataArray.length
            },
            next: function () {
                return dataArray[iterationCount++]
                }
            }
        }

    exports.not = not;
    exports.identity = identity;
    exports.isEven = isEven;
    exports.isGreaterThan = isGreaterThan;
    exports.digitize = digitize;
    exports.modulus = modulus;
    exports.select = select;
    exports.compose = compose;
    exports.concat = concat;
    exports.transduceAsyncIterator = transduceAsyncIterator;
    exports.transduceAsyncHasNextIterator = transduceAsyncHasNextIterator;
    exports.transduceArray = transduceArray;
    exports.mapping = mapping;
    exports.filtering = filtering;
    exports.take = take;
    exports.skip = skip;
    exports.eventing = eventing;
    exports.makeArrayIterator = makeArrayIterator;
    exports.makeAsyncRangeIterator = makeAsyncRangeIterator;
    exports.makeAsyncHasNextRangeIterator = makeAsyncHasNextRangeIterator;

    Object.defineProperty(exports, '__esModule', { value: true });

}));
//# sourceMappingURL=funprog.umd.js.map
